pro MSP_BDP_P3_kokepeli_analysis, Event, input_pathname, output_summary, ierr
;
;   Evaluates a CDS image to test for shifts of the Kokepeli crack in each
;   of the four quadrants
;
;   This version runs on a single image
;
;   for now, subframe images are not tested (they are skipped over)
;
;   DPC 20050621
;       20050726 modified to use convolved input image (sharpened), with positive signal
;       20050727 copied from test_kokepeli.pro and added kernel convolution and file looping
;       20050728 modified to use only the center part of the image (where the crack is located)
;       20050802 modified to output arbitrarily long pathnames to image files
;       20050908 modified to permit calling by batch.pro
;       20051026 mofidied to respedt NOGUI switch
;	    20071122 copied from kokopeli_analysis.pro and cut down to one image
;-----
;
ierr = 0
debug = 0
;
;   do some start-up stuff, like build the convolution kernel
;
kernel = make_kokepeli_kernel(empty)
;

array = double(readfits(input_pathname, header,/SILENT))
asz = size(array)
ncol = asz[1]
nrow = asz[2]
if(ncol ne 1024 or nrow ne 1026) then begin
    MSP_BDP_print_text, Event, "Skipping Image "+input_pathname+" with Ncol, Nrow = "+$
    		STRING(format='(i4,i4)',ncol, nrow)
    		output_summary = {ierr: ierr}
    		ierr = 1
    goto, skip
endif
;
;   extract the center portion containing the crack
;
x1 = 350
x2 = 550
y1 = 160
y2 = 800
array = array[x1:x2,y1:y2]
;
;  perform the convolution with the kokepeli RH kernel
;
convolved_array = kokepeli_sharpen(array, kernel)
;
;  extract the pixels near the kokepeli crack in each quad
;
extracted_data = kokepeli_data_extract(convolved_array,x1,y1)
;
;   data array has format (3,4,21,21) = (Z:X:Y, quad, IX, IY)
;
sz = size(extracted_data)
;
;   find initial guess to maximum in each column
;
vector = fltarr(21)
;
gfit = fltarr(4,3,21)     ; quad,  row-value/row-uncertainty/column-value, column-index
;
;   for each quad
;
for iquad = 0, 3 do begin
    ;
    ;  for each column
    ;
    for icol = 0, 20 do begin
        ;
        ; pull a column of pixels from the array
        ;
        vector = reform(extracted_data[0,iquad,icol,*])
        ;
        ; find its maximum
        ;
        vmax = kokepeli_maximum_detector(vector)
        ;
        ; report the maximum
        ;
        xmax = extracted_data[1,iquad,icol,vmax]
        ymax = extracted_data[2,iquad,icol,vmax]
        ;
        ;    fit the maximum with a gaussian
        ;
        gmax = kokepeli_gaussian_fit(vector,vmax,ymax)
        ;
        ;    load results array
        ;
        gfit[iquad,0,icol] = gmax[0] + ymax - vmax   ; y = row
        gfit[iquad,1,icol] = gmax[1]           ; uncertainty in row
        gfit[iquad,2,icol] = xmax              ; x = column number
        ;
    endfor
endfor
;
;   plot results
;
;if(debug ne 0) then begin
;    for iquad = 0, 3 do begin
;        window, 11
;        x = fltarr(21)
;        y = fltarr(21)
;        sy = fltarr(21)
;        x = reform(gfit[iquad,2,*])
;        y = reform(gfit[iquad,0,*])
;        sy = reform(gfit[iquad,1,*])
;        plot, x, y, yrange=[min(y),max(y)], psym=4
;        sy10 = 10.0 * sy
;        oploterr, x, y, sy10, 3
;    ;
;        cursor, xx, yy, /WAIT
;        wait, 0.5
;        wdelete, 11
;    endfor
;endif
;
;   line fit results
;
constants = kokepeli_offset_fitter(gfit,debug)
;
;   compare Y-jumps at break (column 511.5)
;
y_UL = constants[0,1,0] + 511.5 * constants[0,0,0]
y_UR = constants[1,1,0] + 511.5 * constants[1,0,0]
y_LL = constants[2,1,0] + 511.5 * constants[2,0,0]
y_LR = constants[3,1,0] + 511.5 * constants[3,0,0]
;
a_UL = constants[0,1,0]
a_UR = constants[1,1,0]
a_LL = constants[2,1,0]
a_LR = constants[3,1,0]
;
;   uncertainties
;
sa_UL = constants[0,1,1]
sa_UR = constants[1,1,1]
sa_LL = constants[2,1,1]
sa_LR = constants[3,1,1]
;
;   compare offset term from ideal ones
;
;   uppers
;
upper_const = -172.0
lower_const = -454.0
slope = -1.759259
;
;   recommend shifts or not
;
UL = 0
UR = 0
LL = 0
LR = 0
;
if(abs(a_UL - upper_const) lt 0.75) then begin
    ;print, "UL quad within 0.75 pix of ideal - no shift needed"
endif else begin
    del_const = (upper_const - a_UL)/slope
    UL = round(del_const)
    ;print, "UL quad off by ",del_const," from ideal - shift required"
endelse
;
if(abs(a_UR - upper_const) lt 0.75) then begin
    ;print, "UR quad within 0.75 pix of ideal - no shift needed"
endif else begin
    del_const = (upper_const - a_UR)/slope
    UR = round(del_const)
    ;print, "UR quad off by ",del_const," from ideal - shift required"
endelse
;
if(abs(a_LL - lower_const) lt 0.75) then begin
    ;print, "LL quad within 0.75 pix of ideal - no shift needed"
endif else begin
    del_const = (lower_const - a_LL)/slope
    LL = round(del_const)
    ;print, "LL quad off by ",del_const," from ideal - shift required"
endelse
;
if(abs(a_LR - lower_const) lt 0.75) then begin
    ;print, "LR quad within 0.75 pix of ideal - no shift needed"
endif else begin
    del_const = (lower_const - a_LR)/slope
    LR = round(del_const)
    ;print, "LR quad off by ",del_const," from ideal - shift required"
endelse
;
;   assess uncertainties
;
;     if offset uncertainty under 0.025 pixels, report "A"
;                                 0.05  pixels, report "B"
;                                0.1 pixels, report "C"
;     else, report "D"
;
sA = 0.025
sB = 0.050
sC = 0.100
sLL = 'D'
if(sa_LL lt sC) then sLL = 'C'
if(sa_LL lt sB) then sLL = 'B'
if(sa_LL lt sA) then sLL = 'A'
;
sLR = 'D'
if(sa_LR lt sC) then sLR = 'C'
if(sa_LR lt sB) then sLR = 'B'
if(sa_LR lt sA) then sLR = 'A'
;
sUR = 'D'
if(sa_UR lt sC) then sUR = 'C'
if(sa_UR lt sB) then sUR = 'B'
if(sa_UR lt sA) then sUR = 'A'
;
sUL = 'D'
if(sa_UL lt sC) then sUL = 'C'
if(sa_UL lt sB) then sUL = 'B'
if(sa_UL lt sA) then sUL = 'A'
;
;   return results in a structure
;
sum_abs_shift = abs(LL) + abs(LR) + abs(UL) + abs(UR)
if(sum_abs_shift eq 0) then k_shift = 0 else k_shift = 1
;
output_summary = {k_shift: k_shift, LL: LL, LR: LR, UL: UL, UR: UR, sLL: sLL, sLR: sLR, sUL: sUL, sUR: sUR}
;
;   come here if skipping image
;
skip:
;
end