function bdp_ski_jump_fit_sub, image_input
;
;   fits ski jump for an entire image, one column
;   at a time, returns residuals as an image
;
;   DPC 20050613     original code
;       20060129     modified to work with subframing
;	    20091130	 added bad pixel detection and sidelining, to account for new col clock detection in Koko step
;
image = image_input
;
isize = size(image)
im_col = isize[1]
im_row = isize[2]
;
itype = size(image, /type)
if(itype ne 2 and itype ne 3) then begin
	print, "ERROR - called ski jump fit sub with wrong image type"
	stop
endif
;
if(itype eq 2) then begin
	int_bad_pixel_value = -1000000		; as an integer
endif else begin
	int_bad_pixel_value = -1000000L
endelse
flt_bad_pixel_value = -1.0e6
dbl_bad_pixel_value = -1.0d6
;
center_y = (im_row - 2) / 2
;
if (im_col eq 1024) and (im_row eq 1026) then begin
    ;
    ;   full frame; establish 4 zones and boundaries
    ;
    nzones = 4
    fit_region = intarr(2,nzones)
    fit_order = intarr(nzones)
    apply_region = intarr(2,nzones)     ; [start:stop, zone number]
    ;
    fit_region[0,0] = 0
    fit_region[1,0] = 10
    apply_region[0,0] = 0
    apply_region[1,0] = 8
    fit_order[0] = 6
    ;
    fit_region[0,1] = 6
    fit_region[1,1] = 80
    apply_region[0,1] = 9
    apply_region[1,1] = 60
    fit_order[1] = 8
    ;
    fit_region[0,2] = 50
    fit_region[1,2] = 350
    apply_region[0,2] = 61
    apply_region[1,2] = 300
    fit_order[2] = 6
    ;
    fit_region[0,3] = 250
    fit_region[1,3] = 511
    apply_region[0,3] = 301
    apply_region[1,3] = 511
    fit_order[3] = 2
endif else begin
    ;
    ;  sub frame; details depend on row number
    ;
    if(im_row ge 256) then begin
        nzones = 3
        fit_region = intarr(2,nzones)
        fit_order = intarr(nzones)
        apply_region = intarr(2,nzones)
        ;
        fit_region[0,0] = 0
        fit_region[1,0] = 12
        apply_region[0,0] = 0
        apply_region[1,0] = 8
        fit_order[0] = 6
        ;
        fit_region[0,1] = 8
        fit_region[1,1] = 35
        apply_region[0,1] = 9
        apply_region[1,1] = 28
        fit_order[1] = 6
        ;
        fit_region[0,2] = 30
        fit_region[1,2] = center_y-2
        apply_region[0,2] = 29
        apply_region[1,2] = center_y - 1
        fit_order[2] = 2
    endif else begin
        nzones = 3
        fit_region = intarr(2,nzones)
        fit_order = intarr(nzones)
        apply_region = intarr(2,nzones)
        ;
        fit_region[0,0] = 0
        fit_region[1,0] = 12
        apply_region[0,0] = 0
        apply_region[1,0] = 8
        fit_order[0] = 6
        ;
        fit_region[0,1] = 8
        fit_region[1,1] = 20
        apply_region[0,1] = 9
        apply_region[1,1] = 18
        fit_order[1] = 6
        ;
        fit_region[0,2] = 18
        fit_region[1,2] = center_y-2
        apply_region[0,2] = 19
        apply_region[1,2] = center_y - 1
        fit_order[2] = 2
    endelse
endelse
;
;print, "Nzones = ",nzones
;
fixed = image * 0.0        ; clear result array
;
;   big loop on the columns in the image
;
column_vec = dblarr(im_row)
for icol = 0, im_col-1 do begin
    ;
    column_vec = REFORM(image[icol,0:im_row-1])
    ;
    cut_U = dpc_reverse(column_vec[center_y:im_row-3])   ; swap the upper half order to look like lower half
    cut_L = column_vec[0:center_y-1]
    result = column_vec * 0.0
    ;
    ;   fit each in four zones - as per discovery of shape of ski jump
    ;
    ;   zone 1 fit 4th order poly from 0-10, apply to 0-8
    ;   zone 2 fit 4th order poly from 8-80, apply to 9-60
    ;   zone 3 fit 6th order poly from 50-350, apply to 61-300
    ;   zone 4 fit 2nd order poly from 250-509, apply to 301-511
    ;
    ;---------------------------------------------------------------
    ;
    ;   for subframing, and especially after we tuned the detector in 2005/11,
    ;   probably good enough to fit 0-10 with 4th order, 11-center with 2nd order
    ;
    for iquad = 0, 1 do begin
        ymodel = dblarr(center_y)
        yresid = ymodel
        xmodel = indgen(center_y)
        case iquad of
            0: y = cut_U
            1: y = cut_L
        endcase
        max_x = n_elements(y)-1
       ;
        for izone = 0, nzones-1 do begin
            xfit = fit_region[0,izone] + indgen(fit_region[1,izone] - fit_region[0,izone] + 1)
            y_sub = y[fit_region[0,izone]:fit_region[1,izone]]
            ;
            low_fit = apply_region[0,izone]
	        high_fit = apply_region[1,izone]
            ;
            ;	test that the y values aren't all deleted
            ;
            ind_good = WHERE(y_sub ne dbl_bad_pixel_value, ngood)
            if(ngood gt 0) then begin
	            order = fit_order[izone]
	            ;
	            ;	loop 2 times to reject pixels more than 7 sigma out of mean of region
	            ;
	            for iloop = 0, 1 do begin
	            	yind = WHERE(y_sub ne dbl_bad_pixel_value, nok)
	            	if(nok gt 0) then begin
	            		zz = median_filtered_mean(y_sub[yind])
	            		bad_ind = WHERE(abs(y_sub - zz[0])/zz[1] gt 7.0,n_bad)
	            		if(n_bad gt 0) then begin
	            			y_sub[bad_ind] = dbl_bad_pixel_value
	            		endif
	            	endif
	            endfor
	            ;
	            ;   loop 2 times to reject pixels more than 5 sigma out of mean of fit
	            ;
	            for iloop = 0, 1 do begin
	                yind = where(y_sub ne dbl_bad_pixel_value,nok)
	                if(nok gt 0) then begin
	                	coeffs_sub = POLY_FIT(xfit[yind], y_sub[yind], order, /DOUBLE, YERROR=sigma, $
	                			YFIT=yfit, STATUS=status)
	                	if(status eq 0 or status eq 2) then begin
	                		ybad = where(abs(y_sub[yind] - yfit)/sigma gt 5.,count)
	                		if(count gt 0) then y_sub[ybad] = dbl_bad_pixel_value else goto, out
	                	endif else begin
	                		coeffs_sub = dblarr(order+1) * 0.0
	                	endelse
	                endif else begin
	                	coeffs_sub = dblarr(order+1) * 0.0
	            	endelse
	            endfor
	            out: begin
	               ;print, "Sigma = ",sigma," Coeffs = ",coeffs_sub
	            end
	            ;
	            ;  apply the coefficients to the fit range
	            ;
	            ymodel[low_fit:high_fit] = 0.0d0
	            for ipix = low_fit, high_fit do begin
	                for ipower = 0, order do begin
	                    ymodel[ipix:ipix] = ymodel[ipix:ipix] + coeffs_sub[ipower] * double(xmodel[ipix:ipix])^ipower
	                endfor
	            endfor
	            yresid[low_fit:high_fit] = y[low_fit:high_fit]
	            jind = WHERE(y[low_fit:high_fit] ne dbl_bad_pixel_value,jgood)
	            if(jgood gt 0) then begin
	            	yresid[low_fit+jind] = y[low_fit+jind] - ymodel[low_fit+jind]
	            endif
	        endif else begin
	        	;
	        	;	if deleted, leave them deleted
	        	;
	        	yresid[low_fit:high_fit] = dbl_bad_pixel_value
	        endelse
            ;
        endfor
        ;
        ;   pack into result array
        ;
        case iquad of
            0: begin
                yresid = dpc_reverse(yresid)
                result[center_y:im_row-3] = yresid[0:center_y-1]
               end
            1: result[0:center_y-1] = yresid[0:center_y-1]
        endcase
    endfor
    ;
    result[im_row-2:im_row-1] = dbl_bad_pixel_value
    ;
    fixed[icol,0:im_row-1] = result
    ;
endfor
;
the_end:
;
return, fixed
end
